"""Read in sww file, interpolate at specified locations and
    Cross section at specified T. Cross sections are spatial drawing
    at the specified time.

    '2nd normal' test is employed to find the best fit friction value
    for the given building scenario => 'B_file'

"""
from os import sep
import Numeric
import project
from pyvolution.util import file_function
from create_buildings import WidtH
from pylab import *
from numerical_tools import norm, corr, err  

B_list=['buildings_Orth(45)_D=3_1566.sww',
        'buildings_Orth(45)_D=5_1247.sww',
        'buildings_Orth(45)_D=7_1270.sww',
        'buildings_Orth(45)_D=9_804.sww',
        'buildings_Orth(45)_D=11_844.sww',
        'buildings_Orth(45)_D=13_918.sww',
        'buildings_Orth(45)_D=15_1250.sww',
        'buildings_Orth(45)_D=17_1735.sww']


fd = open("buildings_Orth(45)_report.csv", 'w')
fd.write( 'buildings_Orth(45)_report. Summary at bottom.' + '\n' )


# Building scenario file to be compared to
#---------------------------------------------
#B_list = ['buildings_S0_D=23_1784.sww']
#---------------------------------------------

best_fric=[]
B_Density=[]
best_norm=[]
Breadth=[]


for B_file in B_list:
    swwfile_B = project.outputdir + sep  + B_file
    a=B_file.split('_')
    b=a[2].split('=')
    bre=int(b[1])

    gauge_depth = Numeric.arrayrange((52.5-bre/2), 1.5*WidtH, 25)
    gauge_breadth = 100
    gauge_locations = []

    for GD in gauge_depth:
        gauge_location = [GD,gauge_breadth]
        gauge_locations.append(gauge_location)


    #Read model output
    quantities = ['stage', 'xmomentum', 'ymomentum']

    f_B = file_function(swwfile_B,
                      quantities = quantities,
                      interpolation_points = gauge_locations,
                      verbose = True,
                      use_cache = True)

    fric_OK=[]
    least_norm=[100]
    Normal=[]
    Friction_large = list(Numeric.arange(1,120,1))
    Friction = list(Numeric.arange(0.025,1,0.025))
    #Friction_log = [0.025, 0.1, 0.2, 0.5, 1, 2, 5, 10, 20, 50, 100, 200, 500, 1000, 2000, 5000, 10000]
    Friction.extend(Friction_large)
    for fric in Friction:
        swwfile_F = project.outputdir + sep  + 'friction_n=' + str(fric) + '_3135.sww'
        f_F = file_function(swwfile_F,
                          quantities = quantities,
                          interpolation_points = gauge_locations,
                          verbose = True,
                          use_cache = True)

        t=990 # specifies times to look at cross sections
        
        stages_B = []
        stages_F = []
        vel_B=[]
        vel_F=[]
        for i, GL in enumerate(gauge_depth):

            k = len(gauge_depth) # point of velocity comparision
            # mines data from specified time values in sww file
            # Building file
            wB = f_B(t, point_id = i)[0]
            uhB = f_B(t, point_id = i)[1]
            vhB = f_B(t, point_id = i)[2]
            # Friction file
            wF = f_F(t, point_id = i)[0]        
            uhF = f_B(t, point_id = i)[1]
            vhF = f_B(t, point_id = i)[2]

            #m = sqrt(uh*uh + vh*vh)   #Absolute momentum
            velB = sqrt(uhB*uhB + vhB*vhB) / (wB + 1.e-30) #Absolute velocity
            velF = sqrt(uhF*uhF + vhF*vhF) / (wF + 1.e-30) #Absolute velocity                   
            stages_B.append(wB)
            stages_F.append(wF)


        vel_B.append(velB)
        vel_F.append(velF)
        Ar_B = Numeric.array(stages_B)
        Ar_F = Numeric.array(stages_F)
        diff = abs(Ar_B - Ar_F)
        norm_F = err(Ar_F, Ar_B, 2, relative = True)  # 2nd norm (rel. RMS) test
        Normal.append(norm_F)
        #print " "
        #print norm_F, "2nd norm test result (N_F)"
        #print " "
        if norm_F < least_norm:
            least_norm = norm_F
            fric_OK = fric
            
        else:
            print fric, " NOT OK" 

    
    print fric_OK, " <- This Manning's n is best fit from 2nd norm test"
    print least_norm, "2nd normal of Friction vs Buildings  "
    best_fric.append(fric_OK)
    B_Density.append((float(bre)**2)/625)
    best_norm.append(least_norm)
    Breadth.append(bre)
    
    print vel_F, "friction"
    print vel_B, "Buildings"
    fd.write('_______________________________________________________' + '\n' )
    fd.write( '<<<< Mannings n for [' + B_file + '] >>>>' + '\n' )
    fd.write( '(n), 2nd norm ' + '\n')
    for j,drt in enumerate(Friction):
        fd.write(str(Friction[j]) + ', ' +  str(Normal[j]) + '\n')
         
print best_fric, " << Best fit friction"
print B_Density, " << Building footprint density"
print best_norm, " << Matching normal densities"


len_file=Numeric.arrayrange(0,len(B_list),1)
fd.write('_______________________________________________________' + '\n' )
fd.write( '<<<<< buildings_Orth(45)_report Summary >>>>>' + '\n' )
fd.write( 'width, footprint ratio, mannings n, 2nd norm ' + '\n')
for i in len_file:
    fd.write( str(Breadth[i]) + ', ' + str(B_Density[i]) + ', ' +  str(best_fric[i]) + ', ' +  str(best_norm[i]) + '\n')
fd.close

